import {
    Http, IDictionary, IEventParam,
    MemoryDatasource, Metadata,
} from '@zeedhi/core';
import {
    DialogService, LoadingService, Modal, ModalService, TextInput,
} from '@zeedhi/common';
import { TekGrid } from '@zeedhi/teknisa-components-common';
import { Utilities } from '@/util/Utilities';

export class Fm00040002TopicosAuditoriaController {
    public modalErroExcluirTopicos!: Modal;

    public modalErroInativarTopicos!: Modal;

    public parametrizacaoNota: Boolean = false;

    constructor() {
        this.criarModais();
    }

    public async criarModais() {
        this.modalErroExcluirTopicos = await ModalService.createFromJson('fm_00040_000_auditoria/modal/modal_exclusao_topicos', true);
        this.modalErroInativarTopicos = await ModalService.createFromJson('fm_00040_000_auditoria/modal/modal_inatividade_topicos', true);
    }

    public async abrirModalErro() {
        if (this.modalErroExcluirTopicos) {
            await this.modalErroExcluirTopicos.show();
        } else {
            console.error("Modal 'modalErroExcluirTopicos' não foi inicializado.");
        }
    }

    public fecharModalErro() {
        if (this.modalErroExcluirTopicos) {
            this.modalErroExcluirTopicos.hide();
        }
    }

    public async abrirModalErroInatividade() {
        if (this.modalErroInativarTopicos) {
            await this.modalErroInativarTopicos.show();
        } else {
            console.error("Modal 'modalErroInativarTopicos' não foi inicializado.");
        }
    }

    public fecharModalErroInatividade() {
        if (this.modalErroInativarTopicos) {
            this.modalErroInativarTopicos.hide();
        }
    }

    public excluirTopicos({ event }: any) {
        Utilities.showConfirmDeleteMessage(event, 'Fm00040002TopicosAuditoriaController.aposConfirmarDelete', 'Excluir Registro(s)?');
    }

    public aposConfirmarDelete() {
        const component = Metadata.getInstance<TekGrid>('tek-grid-topicos');
        LoadingService.show();
        Http.post('/excluirRegistros', {
            TOPICOS: component.selectedRows.map((element: any) => element.NRTOPICOAUDIT),
        }).then((response: IDictionary) => {
            if (response.data.status === 'erro') {
                this.montarModalErro(response.data.nao_excluidos);
                Utilities.showSuccessMessage();
                component.datasource.get();
            } else {
                Utilities.showSuccessMessage();
                component.datasource.get();
            }
        }).catch((error: any) => {
            Utilities.showErrorMessage(error);
        }).finally(() => {
            component.selectedRows = [];
            LoadingService.hide();
            DialogService.hide();
        });
    }

    public async montarModalErro(relacionados: any) {
        await this.abrirModalErro();

        const grid = Metadata.getInstance<TekGrid>('grid-topicos-conflito');
        grid.datasource.data = [];
        (grid.datasource as MemoryDatasource).allData = [];

        relacionados.forEach((rel: any) => {
            const dados = {
                NRMODELOAUDIT: rel.NRMODELOAUDIT,
                NMMODELOAUDIT: rel.NMMODELOAUDIT,
            };
            grid.datasource.post(dados);
        });

        await grid.datasource.get();
    }

    public async montarModalErroInativar(relacionados: any) {
        await this.abrirModalErroInatividade();

        const grid = Metadata.getInstance<TekGrid>('grid-topicos-conflito-inativ');
        grid.datasource.data = [];
        (grid.datasource as MemoryDatasource).allData = [];

        relacionados.forEach((rel: any) => {
            const dados = {
                NRMODELOAUDIT: rel.NRMODELOAUDIT,
                NMMODELOAUDIT: rel.NMMODELOAUDIT,
            };
            grid.datasource.post(dados);
        });

        await grid.datasource.get();
    }

    public addNewRow({ component }: IEventParam<TekGrid>) {
        const newId = 0;
        component.addNewRow({ NRTOPICOAUDIT: newId }, 'start');
        setTimeout(() => {
            const nameInput = Metadata.getInstance<TextInput>('NMTOPICOAUDIT_editable_0');
            nameInput.setFocus();
        }, 100);
    }

    /* eslint-disable */
    public canEditRow({ row }: any) {
        return row['__added_row'] === true;
    }
    /* eslint-enable */

    public salvarTopicosAuditoria({ component, event }: any) {
        event.preventDefault();
        const topicos = component.getEditedRows();

        if (topicos.length > 0) {
            if (topicos[0].NMTOPICOAUDIT != null) {
                LoadingService.show();
                Http.post('/salvarTopicosAuditoria', topicos).then((response: IDictionary) => {
                    if (response.data.status === 'erro') {
                        this.montarModalErroInativar(response.data.relacionados);
                    } else {
                        component.datasource.get();
                        Utilities.showSuccessMessage();
                    }
                }).catch((error: any) => {
                    Utilities.showErrorMessage(error);
                }).finally(() => {
                    component.cancelEditedRows();
                    LoadingService.hide();
                });
            } else {
                Utilities.alertMessage('Preencha o campo de "Nome"', 'error');
            }
        } else {
            Utilities.alertMessage('Preencha o campo de "Nome"!', 'error');
        }
    }

    public getParametrizacao() {
        Http.get('/getParametrizacaoCriterios').then((response: any) => {
            this.parametrizacaoNota = (response.data[0].IDUTILIZANOTAAUDIT === 'S');
        });
    }
}
